@@ -202,81 +202,110 @@ void FCWT::daughter_wavelet_multiplication(fftwf_complex *input, fftwf_complex *
     int endpoint = ((int)endpointf);
     int endpoint4 = endpoint>>2;
 
-    // std::cout << "endpoint: " << endpoint << std::endl;
-    // std::cout << "endpoint4: " << endpoint4 << std::endl;
-    // std::cout << "scale: " << scale << std::endl;
-    
-    __m256* O8 = (__m256*)output;
-    __m256* I8 = (__m256*)input;
-    __m256 step4 = _mm256_set1_ps(step);
-    __m256 offset = _mm256_set_ps(3,3,2,2,1,1,0,0);
-    __m256 maximum = _mm256_set1_ps(isizef-1);
-    
-    int athreads = min(threads,max(1,endpoint4/16));
-    int batchsize = (endpoint4/athreads);
-    int s4 = (isize>>2)-1;
-
-    #pragma omp parallel for
-    for(int i=0; i<athreads; i++) {
-        int start = batchsize*i;
-        int end = batchsize*(i+1);
+    #ifdef AVX
+        //has avx instructions
+        __m256* O8 = (__m256*)output;
+        __m256* I8 = (__m256*)input;
+        __m256 step4 = _mm256_set1_ps(step);
+        __m256 offset = _mm256_set_ps(3,3,2,2,1,1,0,0);
+        __m256 maximum = _mm256_set1_ps(isizef-1);
         
-        for(int q4=start; q4<end; q4++) {
-            float q = (float)q4*4;
+        int athreads = min(threads,max(1,endpoint4/16));
+        int batchsize = (endpoint4/athreads);
+        int s4 = (isize>>2)-1;
 
-            __m256 qq = _mm256_set1_ps(q);
+
+        #pragma omp parallel for
+        for(int i=0; i<athreads; i++) {
+            int start = batchsize*i;
+            int end = batchsize*(i+1);
             
-            U256f tmp = {_mm256_min_ps(maximum,_mm256_mul_ps(step4,_mm256_add_ps(qq,offset)))};
-
-            __m256 wav = _mm256_set_ps(
-                                       mother[(int)tmp.a[7]]*(1-2*imaginary),
-                                       mother[(int)tmp.a[6]],
-                                       mother[(int)tmp.a[5]]*(1-2*imaginary),
-                                       mother[(int)tmp.a[4]],
-                                       mother[(int)tmp.a[3]]*(1-2*imaginary),
-                                       mother[(int)tmp.a[2]],
-                                       mother[(int)tmp.a[1]]*(1-2*imaginary),
-                                       mother[(int)tmp.a[0]]);
-            
-            if(imaginary) {
-                __m256 tmp2 = _mm256_mul_ps(I8[q4],wav);
-                O8[q4] = _mm256_shuffle_ps(tmp2, tmp2, 177);
-            } else {
-                O8[q4] = _mm256_mul_ps(I8[q4],wav);
-            }
-        }
-
-        if(doublesided) {
             for(int q4=start; q4<end; q4++) {
-                float q = (float)(q4*4);
-                
-                __m256 qq = _mm256_set1_ps(q);
+                float q = (float)q4*4;
 
-                U256f tmp = {_mm256_mul_ps(step4,_mm256_add_ps(qq,offset))};
+                __m256 qq = _mm256_set1_ps(q);
                 
+                U256f tmp = {_mm256_min_ps(maximum,_mm256_mul_ps(step4,_mm256_add_ps(qq,offset)))};
+                //U256f tmp = {_mm256_mul_ps(step4,_mm256_add_ps(qq,offset))};
+
                 __m256 wav = _mm256_set_ps(
-                                           mother[(int)tmp.a[0]]*(1-2*imaginary),
-                                           mother[(int)tmp.a[1]],
-                                           mother[(int)tmp.a[2]]*(1-2*imaginary),
-                                           mother[(int)tmp.a[3]],
-                                           mother[(int)tmp.a[4]]*(1-2*imaginary),
-                                           mother[(int)tmp.a[5]],
-                                           mother[(int)tmp.a[6]]*(1-2*imaginary),
-                                           mother[(int)tmp.a[7]]);
+                                        mother[(int)tmp.a[7]]*(1-2*imaginary),
+                                        mother[(int)tmp.a[6]],
+                                        mother[(int)tmp.a[5]]*(1-2*imaginary),
+                                        mother[(int)tmp.a[4]],
+                                        mother[(int)tmp.a[3]]*(1-2*imaginary),
+                                        mother[(int)tmp.a[2]],
+                                        mother[(int)tmp.a[1]]*(1-2*imaginary),
+                                        mother[(int)tmp.a[0]]);
                 
                 if(imaginary) {
-                    __m256 tmp2 = _mm256_mul_ps(I8[s4-q4],wav);
-                    O8[s4-q4] = _mm256_shuffle_ps(tmp2, tmp2, 177);
+                    __m256 tmp2 = _mm256_mul_ps(I8[q4],wav);
+                    O8[q4] = _mm256_shuffle_ps(tmp2, tmp2, 177);
                 } else {
-                    O8[s4-q4] = _mm256_mul_ps(I8[s4-q4],wav);
+                    O8[q4] = _mm256_mul_ps(I8[q4],wav);
+                }
+            }
+
+            if(doublesided) {
+                for(int q4=start; q4<end; q4++) {
+                    float q = (float)(q4*4);
+                    
+                    __m256 qq = _mm256_set1_ps(q);
+                    U256f tmp = {_mm256_mul_ps(step4,_mm256_add_ps(qq,offset))};
+                    
+                    __m256 wav = _mm256_set_ps(
+                                            mother[(int)tmp.a[0]]*(1-2*imaginary),
+                                            mother[(int)tmp.a[1]],
+                                            mother[(int)tmp.a[2]]*(1-2*imaginary),
+                                            mother[(int)tmp.a[3]],
+                                            mother[(int)tmp.a[4]]*(1-2*imaginary),
+                                            mother[(int)tmp.a[5]],
+                                            mother[(int)tmp.a[6]]*(1-2*imaginary),
+                                            mother[(int)tmp.a[7]]);
+                    
+                    if(imaginary) {
+                        __m256 tmp2 = _mm256_mul_ps(I8[s4-q4],wav);
+                        O8[s4-q4] = _mm256_shuffle_ps(tmp2, tmp2, 177);
+                    } else {
+                        O8[s4-q4] = _mm256_mul_ps(I8[s4-q4],wav);
+                    }
                 }
             }
         }
-    }
+    #else
+        int athreads = min(threads,max(1,endpoint/16));
+        int batchsize = (endpoint/athreads);
+        float maximum = isizef-1;
+        int s1 = isize-1;
+
+        #pragma omp parallel for
+        for(int i=0; i<athreads; i++) {
+            int start = batchsize*i;
+            int end = batchsize*(i+1);
+            
+            for(int q1=start; q1<end; q1++) {
+                float q = (float)q1;
+                float tmp = min(maximum,step*q);
+                
+                output[q1][0] = input[q1][0]*mother[(int)tmp];
+                output[q1][1] = input[q1][1]*mother[(int)tmp]*(1-2*imaginary);
+            }
+
+            if(doublesided) {
+                for(int q1=start; q1<end; q1++) {
+                    float q = (float)q1;
+                    float tmp = min(maximum,step*q);
+                    
+                    output[s1-q1][0] = input[s1-q1][0]*mother[(int)tmp]*(1-2*imaginary);
+                    output[s1-q1][1] = input[s1-q1][1]*mother[(int)tmp];
+                }
+            }
+        }
+
+    #endif
     
     return;
 }
-
 void FCWT::create_FFT_optimization_plan(int maxsize, int flags) {
     
     int nt = find2power(maxsize);
@@ -410,6 +439,12 @@ void FCWT::cwt(float *pinput, int psize, complex<float>* poutput, Scales *scales
         Ihat = (fftwf_complex*)aligned_alloc(32, newsize*sizeof(fftwf_complex));
         O1 = (fftwf_complex*)aligned_alloc(32, newsize*sizeof(fftwf_complex));
     #endif
+
+    #ifdef AVX
+        std::cout << "AVX support detected" << std::endl;
+    #else
+        std::cout << "AVX support not detected" << std::endl;
+    #endif
     
     //Copy input to new input buffer
     memcpy(input,pinput,sizeof(float)*size);
